@@ -8,6 +8,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "soup-address.h"
 #include "soup-auth.h"
 #include "soup-enum-types.h"
 #include "soup-marshal.h"
@@ -145,6 +146,8 @@ finalize (GObject *object)
 
 	if (priv->uri)
 		soup_uri_free (priv->uri);
+	if (priv->addr)
+		g_object_unref (priv->addr);
 
 	if (priv->auth)
 		g_object_unref (priv->auth);
@@ -554,7 +557,7 @@ get_property (GObject *object, guint prop_id,
  * Creates a new empty #SoupMessage, which will connect to @uri
  *
  * Return value: the new #SoupMessage (or %NULL if @uri could not
- * be parsed or is not a valid HTTP/HTTPS URI).
+ * be parsed).
  */
 SoupMessage *
 soup_message_new (const char *method, const char *uri_string)
@@ -568,6 +571,10 @@ soup_message_new (const char *method, const char *uri_string)
 	uri = soup_uri_new (uri_string);
 	if (!uri)
 		return NULL;
+	if (!uri->host) {
+		soup_uri_free (uri);
+		return NULL;
+	}
 
 	msg = soup_message_new_from_uri (method, uri);
 	soup_uri_free (uri);
@@ -579,20 +586,13 @@ soup_message_new (const char *method, const char *uri_string)
  * @method: the HTTP method for the created request
  * @uri: the destination endpoint (as a #SoupURI)
  * 
- * Creates a new empty #SoupMessage, which will connect to @uri.
+ * Creates a new empty #SoupMessage, which will connect to @uri
  *
- * Return value: the new #SoupMessage (or %NULL if @uri is not a
- * valid HTTP/HTTPS URI)
+ * Return value: the new #SoupMessage
  */
 SoupMessage *
 soup_message_new_from_uri (const char *method, SoupURI *uri)
 {
-	g_return_val_if_fail (method != NULL, NULL);
-	g_return_val_if_fail (uri != NULL, NULL);
-
-	if (!SOUP_URI_VALID_FOR_HTTP (uri))
-		return NULL;
-
 	return g_object_new (SOUP_TYPE_MESSAGE,
 			     SOUP_MESSAGE_METHOD, method,
 			     SOUP_MESSAGE_URI, uri,
@@ -1269,9 +1269,8 @@ soup_message_is_keepalive (SoupMessage *msg)
  * @msg: a #SoupMessage
  * @uri: the new #SoupURI
  *
- * Sets @msg's URI to @uri, which must be a valid HTTP/HTTPS URI (per
- * SOUP_URI_VALID_FOR_HTTP()). If @msg has already been sent and you
- * want to re-send it with the new URI, you need to call
+ * Sets @msg's URI to @uri. If @msg has already been sent and you want
+ * to re-send it with the new URI, you need to call
  * soup_session_requeue_message().
  **/
 void
@@ -1280,11 +1279,14 @@ soup_message_set_uri (SoupMessage *msg, SoupURI *uri)
 	SoupMessagePrivate *priv;
 
 	g_return_if_fail (SOUP_IS_MESSAGE (msg));
-	g_return_if_fail (SOUP_URI_VALID_FOR_HTTP (uri));
-
 	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+
 	if (priv->uri)
 		soup_uri_free (priv->uri);
+	if (priv->addr) {
+		g_object_unref (priv->addr);
+		priv->addr = NULL;
+	}
 	priv->uri = soup_uri_copy (uri);
 
 	g_object_notify (G_OBJECT (msg), SOUP_MESSAGE_URI);
@@ -1306,6 +1308,31 @@ soup_message_get_uri (SoupMessage *msg)
 	return SOUP_MESSAGE_GET_PRIVATE (msg)->uri;
 }
 
+/**
+ * soup_message_get_address:
+ * @msg: a #SoupMessage
+ *
+ * Gets the address @msg's URI points to. After first setting the
+ * URI on a message, this will be unresolved, although the message's
+ * session will resolve it before sending the message.
+ *
+ * Return value: the address @msg's URI points to
+ **/
+SoupAddress *
+soup_message_get_address (SoupMessage *msg)
+{
+	SoupMessagePrivate *priv;
+
+	g_return_val_if_fail (SOUP_IS_MESSAGE (msg), NULL);
+
+	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	if (!priv->addr) {
+		priv->addr = soup_address_new (priv->uri->host,
+					       priv->uri->port);
+	}
+	return priv->addr;
+}
+
 /**
  * soup_message_set_status:
  * @msg: a #SoupMessage
